local Widgets = _Avyr_GUI_Widgets_Lib
local _Questic = _Questic_ADDON_Globals
local insert, remove, wipe, type, select, UIParent, pairs, ipairs, getn, gsub, strsplit, tonumber = table.insert, table.remove, table.wipe, type, select, UIParent, pairs, ipairs, getn, gsub, strsplit, tonumber;
local print, GetItemInfo, GetContainerItemInfo = print, GetItemInfo, GetContainerItemInfo;
_Questic.OptionsMenu = {}

local Window
local CurrentTab = "GeneralBasics"

_Questic.OptionsMenu.Call = function()
    if Window and not Window.Released then
        Window:Release()
        Window = nil;
    else
        Window = Widgets:CreateWindow(UIParent, "Questic Options")
        Window:Size(500, 425)
        Window.OnCloseFunc = function() Window = nil end

        --Window.Pointer = Window;
        insert(Window.LM, "VerifyWindow");
        insert(Window.LM, "TabDisplay");
        insert(Window.LM, "LeftContent");
        insert(Window.LM, "RightContent");
        --insert(Window.LM, "Pointer");
        Window:MakeMovable(true)
        Window.OnMove = function(self)
            local _, _, _, x, y = self:GetPoint(1)
            if _Questic.GetVariable("OptionsMenuPositionx") and _Questic.GetVariable("OptionsMenuPositiony") then
                _Questic.SaveVariable("OptionsMenuPositionx", x)
                _Questic.SaveVariable("OptionsMenuPositiony", y)
            end
        end

        _Questic.OptionsMenu.UpdatePosition()

        Window.TabDisplay = Widgets:CreateContainer(Window)
        Window.TabDisplay:Size(100, Window:GetHeight() - Window.Header:GetHeight() - 2 * Widgets.BorderSize - 3 * Widgets.Padding)
        Window.TabDisplay:SetPoint("TOPLEFT", Window.Header, "BOTTOMLEFT", 0, -Widgets.Padding)
        Window.TabDisplay:AddItems({
            { Type = "LB", Text = "General settings" },
            {
                Type = "DB",
                Text = "Basics",
                Alignment = "LEFT",
                Name = "GeneralBasicsTabSelect",
                Selected = CurrentTab == "GeneralBasics",
                Func = function(self)
                    _Questic.OptionsMenu.Refresh("GeneralBasics")
                end
            },
            { Type = "LB", Text = "Quests", VSpacing = 10 },
            {
                Type = "DB",
                Text = "Basics",
                Alignment = "LEFT",
                Name = "QuestBasicsTabSelect",
                Selected = CurrentTab == "QuestBasics",
                Func = function(self) _Questic.OptionsMenu.Refresh("QuestBasics") end
            },
            {
                Type = "DB",
                Text = "Item bar",
                Alignment = "LEFT",
                Name = "QuestItemBarTabSelect",
                Selected = CurrentTab == "QuestItemBar",
                Func = function(self) _Questic.OptionsMenu.Refresh("QuestItemBar") end
            },
            { Type = "LB", Text = "Gossip", VSpacing = 10 },
            {
                Type = "DB",
                Text = "Basic",
                Alignment = "LEFT",
                Selected = CurrentTab == "GossipBasics",
                Name = "GossipBasicsTabSelect",
                Func = function(self) _Questic.OptionsMenu.Refresh("GossipBasics") end
            },
            {
                Type = "DB",
                Text = "Filter",
                Alignment = "LEFT",
                Selected = CurrentTab == "GossipList",
                Name = "GossipListTabSelect",
                Func = function(self) _Questic.OptionsMenu.Refresh("GossipList") end
            },
            {
                Type = "DB",
                Text = "Profiles",
                Alignment = "LEFT",
                Selected = CurrentTab == "Profile",
                Name = "ProfileTabSelect",
                VSpacing = 10,
                Func = function(self) _Questic.OptionsMenu.Refresh("Profile") end
            },
            {
                Type = "DB",
                Text = "Credits",
                Alignment = "LEFT",
                Selected = CurrentTab == "Credits",
                Name = "CreditsTabSelect",
                VSpacing = 20,
                Func = function(self) _Questic.OptionsMenu.Refresh("Credits") end
            }
        })
    end
    _Questic.OptionsMenu.Refresh(nil, nil, true)
end

local function CheckRouteNameTaken(Name)
    for i, v in pairs(_Questic.RouteData.DefaultRoutes) do if v.Name == Name then return true end end

    for i, v in ipairs(_Questic.GetVariable("QRRouteList")) do if v.Name == Name then return true end end

    return false
end

local RouteCreationDataSet = {
    Name = "New Route",
    Faction = "Horde",
    WarMode = true,
    Note = "My Route",
    StartLevel = 110,
    Specials = "None",
    Quests = {},
    Maps = {}
}
local function AddRemoveRoute(Index, Data)
    if not Index and Data then
        local Name = Data.Name
        local StartIndex = 1
        local SplitName = { strsplit(" ", Name) }
        if gsub(SplitName[getn(SplitName)], "%D", "") == SplitName[getn(SplitName)] then
            StartIndex = tonumber(SplitName[getn(SplitName)])
            Name = SplitName[1]
            for i, v in ipairs(SplitName) do if i > 1 and i < getn(SplitName) then Name = Name .. " " .. v end end
        end
        if CheckRouteNameTaken(Name) then
            for i = StartIndex, StartIndex + 50 do
                if not CheckRouteNameTaken(Name .. " " .. i) then
                    Name = Name .. " " .. i
                    break
                end
            end
            if CheckRouteNameTaken(Name) then return false end
        end

        local QuestData = {}
        for i, v in ipairs(Data.Quests) do insert(QuestData, v) end
        insert(_Questic.GetVariable("QRRouteList"), 1,
            {
                Name = Name,
                Faction = Data.Faction,
                WarMode = Data.WarMode,
                Note = Data.Note,
                StartLevel = Data.StartLevel,
                Specials = Data.Specials,
                Quests = QuestData,
                Maps = Data.Maps
            })
        return 1
    elseif type(Index) == "number" and _Questic.GetVariable("QRRouteList")[Index] then
        remove(_Questic.GetVariable("QRRouteList"), Index)
    end
end

local function ComplileQuestItemBLacklist()
    local ReturnList = {}
    for i, v in pairs(_Questic.GetVariable("QuestItemBlackList")) do insert(ReturnList, i) end
    return ReturnList
end

local SelectedProfile
local CopyProfile
local CurrentGossipData = _Questic.GossipFunctions.FetchLastGossip()
local tempGossip = {}
local SelectedFilterType = "Preset whitelist"
local SelectedFilterIndex
local GossipFilterList = {}
local SelectedItemBarBlacklistItem

-- refresh options tab
local function UpdateGossipFilterOptionsMenu(Type, Index, Refresh)
    if Window and Window.Released then
        return nil
    end
    if CurrentTab == "GossipList" then
        if Window.VerifyWindow then
            Window.VerifyWindow:Release()
            Window.VerifyWindow = nil
        end
        local EditType
        -- when creating new entry to ensure no current entry is overwritten
        if Type == "NewEntry" or Type == "LastEntry" then
            Index = nil
            EditType = Type
            Type = SelectedFilterType
        end

        -- Check for a change in filter type to update the list of filters
        if Type ~= SelectedFilterType then
            SelectedFilterType = Type
            Refresh = true
        end

        -- check if last gossip is av.
        CurrentGossipData = _Questic.GossipFunctions.FetchLastGossip()
        if CurrentGossipData and CurrentGossipData.NPC and CurrentGossipData.NPC ~= "" and CurrentGossipData.Gossip and CurrentGossipData.Gossip ~= "" then
            Window.LeftContent:ChangeItem("GossipAddLastGossip", "Disable", false)
        else
            Window.LeftContent:ChangeItem("GossipAddLastGossip", "Disable", true)
        end

        -- load correct list into list display
        if Refresh then
            Window.LeftContent:ChangeItem("FilterSelect", "Value", SelectedFilterType)
            if SelectedFilterType == "Preset whitelist" then
                wipe(GossipFilterList)
                for i, v in ipairs(_Questic.GossipFunctions.DefaultList) do
                    insert(GossipFilterList, v.NPC .. ": " .. v.Gossip)
                end
                Window.LeftContent:ChangeItem("ListOfFilters", "List", GossipFilterList)
            elseif SelectedFilterType == "User defined whitelist" then
                wipe(GossipFilterList)
                for i, v in ipairs(_Questic.GetVariable("GossipAutoSelectList")) do
                    if v.NPC and v.Gossip then
                        insert(GossipFilterList, v.NPC .. ": " .. v.Gossip)
                    end
                end
                Window.LeftContent:ChangeItem("ListOfFilters", "List", GossipFilterList)
            elseif SelectedFilterType == "User defined blacklist" then
                wipe(GossipFilterList)
                for i, v in ipairs(_Questic.GetVariable("GossipAutoSelectBlackList")) do
                    if v.NPC and v.Gossip then
                        insert(GossipFilterList, v.NPC .. ": " .. v.Gossip)
                    end
                end
                Window.LeftContent:ChangeItem("ListOfFilters", "List", GossipFilterList)
            end
            Window.LeftContent:ChangeItem("ListOfFilters", "Text", SelectedFilterType)
            SelectedFilterIndex = nil
            Window.LeftContent:ChangeItem("ListOfFilters", "Selected", SelectedFilterIndex)
        end

        -- check whether the index of the selected list entry has changed
        if EditType or Index ~= SelectedFilterIndex then
            SelectedFilterIndex = Index
            Window.LeftContent:ChangeItem("ListOfFilters", "Selected", SelectedFilterIndex)
            Refresh = true
        end

        -- input the correct data into edit boxes
        if Refresh then
            wipe(tempGossip)
            if (SelectedFilterIndex and SelectedFilterType ~= "Preset whitelist") or EditType then
                -- creating a temp data set to be saved later
                tempGossip.Filter = "Whitelist"
                if EditType == "LastEntry" then
                    tempGossip.New = true -- force the creating of a new entry when saving the data set
                    tempGossip.NPC = CurrentGossipData.NPC
                    tempGossip.Gossip = CurrentGossipData.Gossip
                elseif EditType == "NewEntry" then
                    tempGossip.New = true -- force the creating of a new entry when saving the data set
                elseif SelectedFilterType == "User defined whitelist" and _Questic.GetVariable("GossipAutoSelectList")[SelectedFilterIndex] then
                    -- will be compared to the filter when saving the data to determine whether the filter type was changed
                    tempGossip.NPC = _Questic.GetVariable("GossipAutoSelectList")[SelectedFilterIndex].NPC
                    tempGossip.Gossip = _Questic.GetVariable("GossipAutoSelectList")[SelectedFilterIndex].Gossip
                    tempGossip.Note = _Questic.GetVariable("GossipAutoSelectList")[SelectedFilterIndex].Note
                    tempGossip.Disable = _Questic.GetVariable("GossipAutoSelectList")[SelectedFilterIndex].Disable
                    tempGossip.Filter = "Whitelist"
                    tempGossip.OFilter = "Whitelist"
                elseif SelectedFilterType == "User defined blacklist" and _Questic.GetVariable("GossipAutoSelectBlackList")[SelectedFilterIndex] then
                    tempGossip.NPC = _Questic.GetVariable("GossipAutoSelectBlackList")[SelectedFilterIndex].NPC
                    tempGossip.Gossip = _Questic.GetVariable("GossipAutoSelectBlackList")[SelectedFilterIndex].Gossip
                    tempGossip.Note = _Questic.GetVariable("GossipAutoSelectBlackList")[SelectedFilterIndex].Note
                    tempGossip.Disable = _Questic.GetVariable("GossipAutoSelectBlackList")[SelectedFilterIndex].Disable
                    tempGossip.Filter = "Blacklist"
                    tempGossip.OFilter = "Blacklist"
                    -- will be compared to the filter when saving the data to determine whether the filter type was changed
                end
                -- input the temp data
                Window.RightContent:ChangeItem("FilterTypeSBItem", "Disable", false)
                Window.RightContent:ChangeItem("FilterTypeSBItem", "Value", tempGossip.Filter)
                Window.RightContent:ChangeItem("NPCNameTBItem", "Value", tempGossip.NPC or "")
                Window.RightContent:ChangeItem("NPCNameTBItem", "Disable", false)
                Window.RightContent:ChangeItem("GossipTBItem", "Value", tempGossip.Gossip or "")
                Window.RightContent:ChangeItem("GossipTBItem", "Disable", false)
                Window.RightContent:ChangeItem("NoteTBItem", "Value", tempGossip.Note or "")
                Window.RightContent:ChangeItem("NoteTBItem", "Disable", false)
                Window.RightContent:ChangeItem("DisableCBItem", "Value", tempGossip.Disable)
                Window.RightContent:ChangeItem("DisableCBItem", "Disable", false)
            elseif SelectedFilterType == "Preset whitelist" and _Questic.GossipFunctions.DefaultList[SelectedFilterIndex] then
                Window.RightContent:ChangeItem("FilterTypeSBItem", "Disable", true)
                Window.RightContent:ChangeItem("FilterTypeSBItem", "Value", "Whitelist")
                local NPCName = _Questic.GossipFunctions.DefaultList[SelectedFilterIndex].NPC
                local Gossip = _Questic.GossipFunctions.DefaultList[SelectedFilterIndex].Gossip
                local Note = _Questic.GossipFunctions.DefaultList[SelectedFilterIndex].Note
                local Categories = _Questic.GossipFunctions.DefaultList[SelectedFilterIndex].Categories
                Window.RightContent:ChangeItem("NPCNameTBItem", "Value", NPCName or "")
                Window.RightContent:ChangeItem("NPCNameTBItem", "Disable", true)
                Window.RightContent:ChangeItem("GossipTBItem", "Value", Gossip or "")
                Window.RightContent:ChangeItem("GossipTBItem", "Disable", true)
                Window.RightContent:ChangeItem("NoteTBItem", "Value", Note or "")
                Window.RightContent:ChangeItem("NoteTBItem", "Disable", true)
                Window.RightContent:ChangeItem("DisableCBItem",
                    "Value",
                    _Questic.GetVariable("GossipAutoSelectDisabledPresets")[NPCName .. Gossip .. Categories])
                Window.RightContent:ChangeItem("DisableCBItem", "Disable", false)
            else
                -- disable everything
                Window.RightContent:ChangeItem("FilterTypeSBItem", "Disable", true)
                Window.RightContent:ChangeItem("FilterTypeSBItem", "Value", "Whitelist")
                Window.RightContent:ChangeItem("NPCNameTBItem", "Value", "")
                Window.RightContent:ChangeItem("NPCNameTBItem", "Disable", true)
                Window.RightContent:ChangeItem("GossipTBItem", "Value", "")
                Window.RightContent:ChangeItem("GossipTBItem", "Disable", true)
                Window.RightContent:ChangeItem("NoteTBItem", "Value", "")
                Window.RightContent:ChangeItem("NoteTBItem", "Disable", true)
                Window.RightContent:ChangeItem("DisableCBItem", "Value", false)
                Window.RightContent:ChangeItem("DisableCBItem", "Disable", true)
                Window.RightContent:ChangeItem("GossipUndoChanges", "Disable", true)
            end
            -- will be enable as soon as data has actually been changed by the user unless its a new entry that can be saved from the get go
            if tempGossip.New then
                Window.RightContent:ChangeItem("GossipUndoChanges", "Disable", true)
                Window.RightContent:ChangeItem("GossipSaveChanges", "Disable", false)
                Window.RightContent:ChangeItem("GossipDeleteChanges", "Disable", false)
                Window.RightContent:ChangeItem("GossipDeleteChanges", "Text", "Clear")
            elseif tempGossip.NPC then
                Window.RightContent:ChangeItem("GossipUndoChanges", "Disable", true)
                Window.RightContent:ChangeItem("GossipSaveChanges", "Disable", true)
                Window.RightContent:ChangeItem("GossipDeleteChanges", "Disable", false)
                Window.RightContent:ChangeItem("GossipDeleteChanges", "Text", "Delete")
            else
                Window.RightContent:ChangeItem("GossipUndoChanges", "Disable", true)
                Window.RightContent:ChangeItem("GossipSaveChanges", "Disable", true)
                Window.RightContent:ChangeItem("GossipDeleteChanges", "Disable", true)
                Window.RightContent:ChangeItem("GossipDeleteChanges", "Text", "Delete")
            end
        end
    end
end

local function UpdateProfileTab()
    if Window.Released then
        return nil
    end
    if CurrentTab == "Profile" then
        Window.RightContent:ChangeItem("ActiveProfileLB", "Text", "Active profile: " .. (_Questic.ProfileManagement("GetActive") or ""))

        Window.LeftContent:ChangeItem("ListOfProfiles", "List", _Questic.ProfileManagement("GetAll"))
        Window.LeftContent:ChangeItem("ListOfProfiles", "Selected", SelectedProfile)

        Window.RightContent:ChangeItem("SelectedProfileLB", "Text", "Selected profile: " .. (SelectedProfile or ""))
        Window.RightContent:ChangeItem("LoadProfileButton", "Disable", not SelectedProfile)
        Window.RightContent:ChangeItem("RenameSelectedProfileTB", "Disable", not SelectedProfile)
        Window.RightContent:ChangeItem("RenameSelectedProfileTB", "Value", SelectedProfile or "")

        Window.RightContent:ChangeItem("SelectSourceCopyProfileSB", "Value", CopyProfile or "")
        Window.RightContent:ChangeItem("SelectSourceCopyProfileSB", "Disable", not SelectedProfile)

        Window.RightContent:ChangeItem("CopySourceProfileButton", "Value", CopyProfile or "")
        Window.RightContent:ChangeItem("CopySourceProfileButton", "Disable", not CopyProfile)

        Window.RightContent:ChangeItem("DeleteProfileDB", "Disable", not SelectedProfile)
    end
end

_Questic.OptionsMenu.UpdatePosition = function()
    if Window and not Window.Released then
        local x, y = _Questic.GetVariable("OptionsMenuPositionx"), _Questic.GetVariable("OptionsMenuPositiony")
        if not x then
            local cx, cy = UIParent:GetCenter()
            x = cx - Window:GetWidth() / 2
        end
        if not y then
            local cx, cy = UIParent:GetCenter()
            y = cy + Window:GetHeight() / 2
        end
        Window:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", x, y)
    end
end

_Questic.OptionsMenu.Refresh = function(Tab, Row, Force)
    if Window and not Window.Released then
        if Window.VerifyWindow then
            Window.VerifyWindow:Release()
            Window.VerifyWindow = nil
        end
        Row = Row or 3

        if type(Tab) == "string" and CurrentTab ~= Tab then
            _Questic.QuestItemBar.ShowPreviewFrame(false)
            Force = true
        end

        if Force then
            Window.TabDisplay:ChangeItem(CurrentTab .. "TabSelect", "Selected", false)
            CurrentTab = Tab or CurrentTab
            Window.TabDisplay:ChangeItem(CurrentTab .. "TabSelect", "Selected", true)

            if not Window.LeftContent then
                Window.LeftContent = Widgets:CreateContainer(Window)
                Window.LeftContent:Size((Window:GetWidth() - Window.TabDisplay:GetWidth() - 2 * Widgets.BorderSize - 4 * Widgets.Padding) / 2,
                    Window:GetHeight() - Window.Header:GetHeight() - 2 * Widgets.BorderSize - 3 * Widgets.Padding)
                Window.LeftContent:SetPoint("TOPLEFT", Window.TabDisplay, "TOPRIGHT", Widgets.Padding, 0)
                Window.LeftContent:Backdrop()
            end
            if not Window.RightContent then
                Window.RightContent = Widgets:CreateContainer(Window)
                Window.RightContent:Size((Window:GetWidth() - Window.TabDisplay:GetWidth() - 2 * Widgets.BorderSize - 4 * Widgets.Padding) / 2,
                    Window:GetHeight() - Window.Header:GetHeight() - 2 * Widgets.BorderSize - 3 * Widgets.Padding)
                Window.RightContent:SetPoint("TOPLEFT",
                    Window.LeftContent,
                    "TOPRIGHT",
                    Widgets.Padding,
                    0)
                Window.RightContent:Backdrop()
            end

            if CurrentTab == "QuestBasics" then
                ----------------------------------------------------------------------------------------------------------------------
                ------------------------------------------- Quest Settings-------------------------------------------------------------
                ----------------------------------------------------------------------------------------------------------------------
                if Row == 3 or Row == 1 then
                    Window.LeftContent:AddItems({
                        {
                            Type = "CB",
                            Text = "Auto-Share",
                            Value = _Questic.GetVariable("AutoShare"),
                            Func = function(self, Value) _Questic.SaveVariable("AutoShare", Value) end,
                            Tooltip = "Questic will automatically share any sharable quests."
                        },
                        { Type = "LB", Text = "Quest auto-accept", VSpacing = 10 },
                        {
                            Type = "CB",
                            Text = "Enable",
                            Value = _Questic.GetVariable("AutoAccept"),
                            Func = function(self, Value)
                                _Questic.SaveVariable("AutoAccept", Value)
                                Window.LeftContent:ChangeItem("AutoAcceptNormal", "Disable", not Value)
                                Window.LeftContent:ChangeItem("AutoAcceptDaily", "Disable", not Value)
                                Window.LeftContent:ChangeItem("AutoAcceptWeekly", "Disable", not Value)
                                Window.LeftContent:ChangeItem("AutoAcceptLowLevel", "Disable", not Value)
                                Window.LeftContent:ChangeItem("AutoAcceptQuestsInBag", "Disable", not Value)
                                Window.LeftContent:ChangeItem("HideBlizzQuestFrame",
                                    "Disable",
                                    not (Value and _Questic.GetVariable("AutoTurnin")))
                            end,
                            Tooltip = "Automatically accept availiable quests when talking to NPCs,other players sharing quests or when other players accept escort quests."
                        },
                        {
                            Type = "CB",
                            Text = "Normal quests",
                            HSpacing = 15,
                            Disable = not _Questic.GetVariable("AutoAccept"),
                            Name = "AutoAcceptNormal",
                            Value = _Questic.GetVariable("AutoAcceptNormal"),
                            Func = function(self, Value) _Questic.SaveVariable("AutoAcceptNormal", Value) end,
                            Tooltip = "Automatically accept normal quests."
                        },
                        {
                            Type = "CB",
                            Text = "Daily quests",
                            HSpacing = 15,
                            Disable = not _Questic.GetVariable("AutoAccept"),
                            Name = "AutoAcceptDaily",
                            Value = _Questic.GetVariable("AutoAcceptDaily"),
                            Func = function(self, Value) _Questic.SaveVariable("AutoAcceptDaily", Value) end,
                            Tooltip = "Automatically accept daily quests."
                        },
                        {
                            Type = "CB",
                            Text = "Weekly quests",
                            HSpacing = 15,
                            Disable = not _Questic.GetVariable("AutoAccept"),
                            Name = "AutoAcceptWeekly",
                            Value = _Questic.GetVariable("AutoAcceptWeekly"),
                            Func = function(self, Value) _Questic.SaveVariable("AutoAcceptWeekly", Value) end,
                            Tooltip = "Automatically accept weekly quests."
                        },
                        {
                            Type = "CB",
                            Text = "Low level quests",
                            HSpacing = 15,
                            Disable = not _Questic.GetVariable("AutoAccept"),
                            Name = "AutoAcceptLowLevel",
                            Value = _Questic.GetVariable("AutoAcceptLowLevel"),
                            Func = function(self, Value) _Questic.SaveVariable("AutoAcceptLowLevel", Value) end,
                            Tooltip = "When multiple quests are available automatically accept low level quests."
                        },
                        {
                            Type = "CB",
                            Text = "Quests in bag",
                            HSpacing = 15,
                            Disable = not _Questic.GetVariable("AutoAccept"),
                            Name = "AutoAcceptQuestsInBag",
                            Value = _Questic.GetVariable("AutoAcceptQuestsInBag"),
                            Func = function(self, Value)
                                _Questic.SaveVariable("AutoAcceptQuestsInBag", Value)
                            end,
                            Tooltip = "Automatically accepts quests given by items in the players bags."
                        }
                    })
                end
                if Row == 3 or Row == 2 then
                    Window.RightContent:AddItems({
                        { Type = "LB", Text = "Quest auto-turnin", VSpacing = 10 },
                        {
                            Type = "CB",
                            Text = "Enable",
                            Value = _Questic.GetVariable("AutoTurnin"),
                            Func = function(self, Value)
                                _Questic.SaveVariable("AutoTurnin", Value)
                                Window.RightContent:ChangeItem("AutoTurninQuestItems", "Disable", not Value)
                                Window.RightContent:ChangeItem("AutoTurninNonQuestItems", "Disable", not Value)
                                Window.RightContent:ChangeItem("AutoturninCurrency", "Disable", not Value)
                                Window.RightContent:ChangeItem("AutoturninMoney", "Disable", not Value)
                            end,
                            Tooltip = "Turn in quests automatically. Quests for which rewards have to be selected a window will be opened to prompt the player to select a reward and turn in the quest with on click."
                        },
                        {
                            Type = "CB",
                            Text = "Quest items",
                            HSpacing = 15,
                            Name = "AutoTurninQuestItems",
                            Value = _Questic.GetVariable("AutoTurninQuestItems"),
                            Disable = not _Questic.GetVariable("AutoTurnin"),
                            Func = function(self, Value)
                                _Questic.SaveVariable("AutoTurninQuestItems", Value)
                            end,
                            Tooltip = "Automatically turn in quest items for completed quests without requiring player verification."
                        },
                        {
                            Type = "CB",
                            Text = "Non-quest items",
                            HSpacing = 15,
                            Name = "AutoTurninNonQuestItems",
                            Value = _Questic.GetVariable("AutoTurninNonQuestItems"),
                            Disable = not _Questic.GetVariable("AutoTurnin"),
                            Func = function(self, Value)
                                _Questic.SaveVariable("AutoTurninNonQuestItems", Value)
                            end,
                            Tooltip = "Automatically turn in items that are not marked as quest items when needed to complete the quest without requiring player verification."
                        },
                        {
                            Type = "CB",
                            Text = "Currency",
                            HSpacing = 15,
                            Name = "AutoturninCurrency",
                            Value = _Questic.GetVariable("AutoturninCurrency"),
                            Disable = not _Questic.GetVariable("AutoTurnin"),
                            Func = function(self, Value) _Questic.SaveVariable("AutoturninCurrency", Value) end,
                            Tooltip = "Automatically turn in currency (not gold) when needed to complete the quest without requiring player confirmation.\n\nFor example \124T1397630:0\124t Order Resources in exchange for \124T1604167:0\124t Seal of Broken Fate."
                        },
                        {
                            Type = "CB",
                            Text = "Money",
                            HSpacing = 15,
                            Name = "AutoturninMoney",
                            Value = _Questic.GetVariable("AutoturninMoney"),
                            Disable = not _Questic.GetVariable("AutoTurnin"),
                            Func = function(self, Value) _Questic.SaveVariable("AutoturninMoney", Value) end,
                            Tooltip = "Automatically turn in gold (money) when needed to complete the quest without requiring player confirmation.\n\nFor example 2,000g in exchange for \124T1604167:0\124t Seal of Broken Fate."
                        }
                    })
                end
            elseif CurrentTab == "QuestItemBar" then
                ----------------------------------------------------------------------------------------------------------------------
                -------------------------------------- Quest item bar Settings--------------------------------------------------------
                ----------------------------------------------------------------------------------------------------------------------
                if Row == 3 or Row == 1 then
                    Window.LeftContent:AddItems({
                        {
                            Type = "CB",
                            Text = "Enable",
                            Value = _Questic.GetVariable("QuestItemBar"),
                            Func = function(self, Value)
                                _Questic.SaveVariable("QuestItemBar", Value)
                                Window.LeftContent:ChangeItem("QuestItemBarGrowthDirection", "Disable", not Value)
                                Window.LeftContent:ChangeItem("QuestItemBarScale", "Disable", not Value)
                                Window.LeftContent:ChangeItem("QuestItemBarShowCD", "Disable", not Value)
                                Window.LeftContent:ChangeItem("QuestItemBarShowCount", "Disable", not Value)
                                Window.LeftContent:ChangeItem("QuestItemBarShowKeyBinding", "Disable", not Value)
                                Window.LeftContent:ChangeItem("QuestItemBarMaxItemsDisplayed", "Disable", not Value)
                                Window.LeftContent:ChangeItem("MoveEnableItemBar", "Disable", not Value)
                                Window.RightContent:ChangeItem("ResetPositionItemBar", "Disable", not Value)
                                Window.RightContent:ChangeItem("QuestItemBarEnableBlackList", "Disable", not Value)
                                Window.RightContent:ChangeItem("QuestItemBlacklistSL", "Disable", not Value or not _Questic.GetVariable("QuestItemBarEnableBlackList"))
                                Window.RightContent:ChangeItem("QuestItemRemoveBtn", "Disable", not Value or not _Questic.GetVariable("QuestItemBarEnableBlackList") or not SelectedItemBarBlacklistItem)
                                Window.RightContent:ChangeItem("AddQuestItemItemBar", "Disable", not Value or not _Questic.GetVariable("QuestItemBarEnableBlackList"))
                            end,
                            Tooltip = "Searches for useable quest items in the players bags and creates a bar for easy use."
                        },
                        {
                            Type = "DB",
                            Text = "Toggle preview",
                            Name = "MoveEnableItemBar",
                            Tooltip = "",
                            Func = function(self)
                                _Questic.QuestItemBar.ShowPreviewFrame()
                            end,
                            Disable = not _Questic.GetVariable("QuestItemBar")
                        },
                        { Type = "LB", Text = "Bar settings", VSpacing = 20 },
                        {
                            Type = "SB",
                            Text = "Growth direction",
                            Name = "QuestItemBarGrowthDirection",
                            Tooltip = "The growth direction of the quest item bar.",
                            Value = _Questic.GetVariable("QuestItemBarGrowthDirection"),
                            Options = { "Right", "Left", "Down", "Up" },
                            Func = function(self, Value, Index)
                                _Questic.SaveVariable("QuestItemBarGrowthDirection", Value)
                            end,
                            Disable = not _Questic.GetVariable("QuestItemBar")
                        },
                        {
                            Type = "NB",
                            Text = "Number of items displayed",
                            Name = "QuestItemBarMaxItemsDisplayed",
                            Tooltip = "Number of quest items displayed (0 for unlimited amount).",
                            Max = 20,
                            Min = 0,
                            Step = 1,
                            Value = _Questic.GetVariable("QuestItemBarMaxItemsDisplayed"),
                            Func = function(self, Value) _Questic.SaveVariable("QuestItemBarMaxItemsDisplayed", Value) end,
                            Disable = not _Questic.GetVariable("QuestItemBar")
                        },
                        { Type = "LB", Text = "Quest item settings", VSpacing = 20 },
                        {
                            Type = "NB",
                            Text = "Scale",
                            Name = "QuestItemBarScale",
                            Tooltip = "Scale of compare quest display.",
                            Max = 2,
                            Min = 0.1,
                            Step = 0.05,
                            Value = _Questic.GetVariable("QuestItemBarScale"),
                            Func = function(self, Value) _Questic.SaveVariable("QuestItemBarScale", Value) end,
                            Disable = not _Questic.GetVariable("QuestItemBar")
                        },
                        {
                            Type = "CB",
                            Text = "Show cooldown",
                            Name = "QuestItemBarShowCD",
                            Value = _Questic.GetVariable("QuestItemBarShowCD"),
                            Func = function(self, Value) _Questic.SaveVariable("QuestItemBarShowCD", Value) end,
                            Disable = not _Questic.GetVariable("QuestItemBar")
                        },
                        {
                            Type = "CB",
                            Text = "Show item count",
                            Name = "QuestItemBarShowCount",
                            Value = _Questic.GetVariable("QuestItemBarShowCount"),
                            Func = function(self, Value) _Questic.SaveVariable("QuestItemBarShowCount", Value) end,
                            Disable = not _Questic.GetVariable("QuestItemBar")
                        },
                        {
                            Type = "CB",
                            Text = "Show key bindings",
                            Name = "QuestItemBarShowKeyBinding",
                            Value = _Questic.GetVariable("QuestItemBarShowKeyBinding"),
                            Func = function(self, Value) _Questic.SaveVariable("QuestItemBarShowKeyBinding", Value) end,
                            Disable = not _Questic.GetVariable("QuestItemBar")
                        }
                    })
                end
                if Row == 3 or Row == 2 then
                    Window.RightContent:AddItems({
                        { Type = "LB", Text = "Positioning" },
                        {
                            Type = "DB",
                            Text = "Reset position",
                            Name = "ResetPositionItemBar",
                            Tooltip = "Resets the position of the quest item bar.",
                            Func = function(self)
                                _Questic.QuestItemBar.Update(true);
                            end,
                            Disable = not _Questic.GetVariable("QuestItemBar")
                        },
                        { Type = "LB", Text = "Quest item blacklist", VSpacing = 20 },
                        {
                            Type = "CB",
                            Text = "Enable",
                            Name = "QuestItemBarEnableBlackList",
                            Value = _Questic.GetVariable("QuestItemBarEnableBlackList"),
                            Func = function(self, Value)
                                _Questic.SaveVariable("QuestItemBarEnableBlackList", Value)
                                Window.RightContent:ChangeItem("AddQuestItemItemBar", "Disable", not _Questic.GetVariable("QuestItemBar") or not Value)
                                Window.RightContent:ChangeItem("QuestItemBlacklistSL", "Disable", not _Questic.GetVariable("QuestItemBar") or not Value)
                                Window.RightContent:ChangeItem("QuestItemRemoveBtn", "Disable", not _Questic.GetVariable("QuestItemBar") or not Value or not SelectedItemBarBlacklistItem)
                            end,
                            Tooltip = "Enable the list of items that will not be displayed on the quest item bar.",
                            Disable = not _Questic.GetVariable("QuestItemBar")
                        },
                        {
                            Type = "SL",
                            Text = "Quest item blacklist",
                            HSpacing = 15,
                            Height = 120,
                            Name = "QuestItemBlacklistSL",
                            Tooltip = "Add quest item to the blacklist to no longer be displayed in the quest item bar.",
                            List = ComplileQuestItemBLacklist(),
                            Selected = SelectedItemBarBlacklistItem,
                            Func = function(self, Value, Name)
                                SelectedItemBarBlacklistItem = Name
                                Window.RightContent:ChangeItem("QuestItemRemoveBtn", "Disable", not _Questic.GetVariable("QuestItemBar") or not _Questic.GetVariable("QuestItemBarEnableBlackList"))
                            end,
                            Disable = not _Questic.GetVariable("QuestItemBar") or not _Questic.GetVariable("QuestItemBarEnableBlackList")
                        },
                        {
                            Type = "DB",
                            HSpacing = 15,
                            Text = "Remove",
                            Name = "QuestItemRemoveBtn",
                            Func = function()
                                if SelectedItemBarBlacklistItem then
                                    if Window.VerifyWindow then
                                        Window.VerifyWindow:Release()
                                        Window.VerifyWindow = nil
                                    end
                                    Window.VerifyWindow = Widgets:VerifyAction(UIParent, "Confirm delete", "Are you sure you want to delete: \"" .. SelectedItemBarBlacklistItem .. "\"?", function()
                                        Window.VerifyWindow = nil
                                        _Questic.QuestItemBlackList("Remove", SelectedItemBarBlacklistItem)
                                        SelectedItemBarBlacklistItem = nil;
                                        Window.RightContent:ChangeItem("QuestItemBlacklistSL", "List", ComplileQuestItemBLacklist())
                                        Window.RightContent:ChangeItem("QuestItemBlacklistSL", "Selected", SelectedItemBarBlacklistItem)
                                        Window.RightContent:ChangeItem("QuestItemRemoveBtn", "Disable", true)
                                    end)
                                end
                            end,
                            Disable = not _Questic.GetVariable("QuestItemBar") or not _Questic.GetVariable("QuestItemBarEnableBlackList") or not SelectedItemBarBlacklistItem,
                        },
                        {
                            Type = "SB",
                            Text = "Add from inventory",
                            HSpacing = 15,
                            Name = "AddQuestItemItemBar",
                            Tooltip = "Remove quest item from the blacklist to again be displayed in the quest item bar.",
                            Func = function(self, Value)
                                _Questic.QuestItemBlackList("Add", Value)
                                Window.RightContent:ChangeItem("QuestItemBlacklistSL", "List", ComplileQuestItemBLacklist())
                                Window.RightContent:ChangeItem("AddQuestItemItemBar", "Value", nil)
                            end,
                            Options = function()
                                local ReturnList = {}
                                for i, v in pairs(_Questic.QuestItemList) do
                                    local QuestItem = GetItemInfo(select(7, GetContainerItemInfo(v.BagID, v.SlotID)))
                                    insert(ReturnList, QuestItem)
                                end
                                return ReturnList
                            end,
                            Disable = not _Questic.GetVariable("QuestItemBar") or not _Questic.GetVariable("QuestItemBarEnableBlackList")
                        },
                        {
                            VSpacing = 20,
                            Alignment = "LEFT",
                            Type = "LB",
                            Text = "Key bindings for the first 6 quest items displayed in the item bar can be assigned in the \"Key Bindings\" options menu under \"Other\"."
                        }
                    })
                end
            elseif CurrentTab == "GossipBasics" then
                ----------------------------------------------------------------------------------------------------------------------
                ------------------------------------------ Gossip Settings------------------------------------------------------------
                ----------------------------------------------------------------------------------------------------------------------
                if Row == 3 or Row == 1 then
                    Window.LeftContent:AddItems({
                        { Type = "LB", Text = "Auto-gossip" },
                        {
                            Type = "CB",
                            Text = "Enable auto-gossip",
                            Value = _Questic.GetVariable("GossipAutoSelect"),
                            Func = function(self, Value)
                                _Questic.SaveVariable("GossipAutoSelect", Value)
                                Window.LeftContent:ChangeItem("GossipAutoSelectPriorSkip", "Disable", not Value)
                                Window.LeftContent:ChangeItem("GossipAutoSelectIgnoreSkip", "Disable", not Value)
                                Window.LeftContent:ChangeItem("GossipAutoSelectSingleOptionGossip",
                                    "Disable",
                                    not Value)
                            end,
                            Tooltip = "Automtically selects gossip if it is part of the auto-gossip list."
                        },
                        {
                            Type = "CB",
                            Text = "Prioritize skip options",
                            Name = "GossipAutoSelectPriorSkip",
                            HSpacing = 15,
                            Value = _Questic.GetVariable("GossipAutoSelectPriorSkip"),
                            Func = function(self, Value)
                                _Questic.SaveVariable("GossipAutoSelectPriorSkip", Value)
                            end,
                            Disable = not _Questic.GetVariable("GossipAutoSelect"),
                            Tooltip = "The addon will prioritize gossip options that skip sections of the story.\n\nFor example when having completed the battle for Lordaeron scenario this becomes skipable which the addon will always do if the option is enabled."
                        },
                        {
                            Type = "CB",
                            Text = "Ignore skip options",
                            Name = "GossipAutoSelectIgnoreSkip",
                            HSpacing = 15,
                            Value = _Questic.GetVariable("GossipAutoSelectIgnoreSkip"),
                            Func = function(self, Value)
                                _Questic.SaveVariable("GossipAutoSelectIgnoreSkip", Value)
                            end,
                            Disable = not _Questic.GetVariable("GossipAutoSelect"),
                            Tooltip = "The addon will ignore gossip options that skip sections of the story.\n\nFor example when having completed the battle for Lordaeron scenario this becomes skipable which the addon will always do if the option is enabled."
                        },
                        {
                            Type = "CB",
                            Text = "Auto select gossip option if only one is available.",
                            Lines = 3,
                            Name = "GossipAutoSelectSingleOptionGossip",
                            HSpacing = 15,
                            Value = _Questic.GetVariable("GossipAutoSelectSingleOptionGossip"),
                            Func = function(self, Value)
                                _Questic.SaveVariable("GossipAutoSelectSingleOptionGossip", Value)
                            end,
                            Disable = not _Questic.GetVariable("GossipAutoSelect"),
                            Tooltip = "If the NPC presents to the player with only one gossip option this one gossip option will be selected."
                        },
                    })
                end
                if Row == 3 or Row == 2 then
                    Window.RightContent:AddItems({
                        {
                            Type = "LB",
                            Text = "Auto-gossip whitelist",
                        },
                        {
                            Type = "DB",
                            Text = "Show auto-gossip whitelist",
                            Name = "ShowGossipWhitelist",
                            Disable = not _Questic.GetVariable("GossipAutoSelect"),
                            Tooltip = "A list of NPCs for which set gossip options are automatically selected.",
                            Func = function(self) _Questic.OptionsMenu.Refresh("GossipList") end
                        },
                        {
                            Type = "CB",
                            Text = "Right-Click gossip add",
                            Value = _Questic.GetVariable("RightClickGossipAdd"),
                            Func = function(self, Value)
                                _Questic.SaveVariable("RightClickGossipAdd", Value)
                            end,
                            Tooltip = "When Right-Clicking a gossip option in the Blizzard or gossip quick select frame it is automatically added to the auto-gossip list."
                        },
                        {
                            Type = "LB",
                            Text = "Auto-gossip blacklist",
                            VSpacing = 20
                        },
                        {
                            Type = "DB",
                            Text = "Show auto-gossip blacklist",
                            Name = "ShowGossipBlacklist",
                            Disable = not _Questic.GetVariable("GossipAutoSelect"),
                            Tooltip = "A list of gossip items for which set gossip options are not automatically selected (The blacklist overrides any other auto-gossip selection).",
                            Func = function(self) _Questic.OptionsMenu.Refresh("GossipList") end
                        },
                        {
                            Type = "CB",
                            Text = "Modified Right-Click gossip blacklist",
                            Lines = 2,
                            Value = _Questic.GetVariable("RightClickGossipBlacklistAdd"),
                            Func = function(self, Value)
                                _Questic.SaveVariable("RightClickGossipBlacklistAdd", Value)
                            end,
                            Tooltip = "When Right-Clicking while pressing a modifier key (alt, shift or ctrl) a gossip option in the Blizzard or gossip quick select frame it is automatically added to gossip blacklist."
                        }
                    })
                end
            elseif CurrentTab == "GossipList" then
                ----------------------------------------------------------------------------------------------------------------------
                --------------------------------------- Gossip List Settings----------------------------------------------------------
                ----------------------------------------------------------------------------------------------------------------------
                if Row == 3 or Row == 1 then
                    Window.LeftContent:AddItems({
                        {
                            Type = "DB",
                            Text = "Create new entry",
                            Name = "GossipCreateNew",
                            Func = function() UpdateGossipFilterOptionsMenu("NewEntry") end
                        },
                        {
                            Type = "DB",
                            Text = "Add last selected gossip",
                            Name = "GossipAddLastGossip",
                            Func = function() UpdateGossipFilterOptionsMenu("LastEntry") end
                        },
                        {
                            VSpacing = 15,
                            Type = "SB",
                            Text = "Filters",
                            Value = SelectedFilterType,
                            Options = {
                                "Preset whitelist",
                                "User defined whitelist",
                                "User defined blacklist"
                            },
                            Name = "FilterSelect",
                            Func = function(_, Value) UpdateGossipFilterOptionsMenu(Value) end
                        },
                        {
                            VSpacing = 15,
                            Type = "SL",
                            Text = "Presets",
                            Height = 260,
                            Name = "ListOfFilters",
                            Func = function(_, Value)
                                UpdateGossipFilterOptionsMenu(SelectedFilterType, Value)
                            end
                        }
                    })
                end
                if Row == 3 or Row == 2 then
                    Window.RightContent:AddItems({
                        {
                            Type = "SB",
                            Text = "Filter type",
                            Options = { "Whitelist", "Blacklist" },
                            Value = "Whitelist",
                            Func = function(_, Value)
                                tempGossip.Filter = Value
                                if SelectedFilterIndex then -- cant undo changes when creating a new item
                                    Window.RightContent:ChangeItem("GossipUndoChanges", "Disable", false)
                                end
                                Window.RightContent:ChangeItem("GossipSaveChanges", "Disable", false)
                            end,
                            Name = "FilterTypeSBItem"
                        },
                        {
                            Type = "TB",
                            Text = "NPC name",
                            Func = function(_, Value)
                                tempGossip.NPC = Value
                                if SelectedFilterIndex then -- cant undo changes when creating a new item
                                    Window.RightContent:ChangeItem("GossipUndoChanges", "Disable", false)
                                end
                                Window.RightContent:ChangeItem("GossipSaveChanges", "Disable", false)
                            end,
                            Name = "NPCNameTBItem",
                            Value = ""
                        },
                        {
                            Type = "TB",
                            Text = "Gossip",
                            Func = function(_, Value)
                                tempGossip.Gossip = Value
                                if SelectedFilterIndex then -- cant undo changes when creating a new item
                                    Window.RightContent:ChangeItem("GossipUndoChanges", "Disable", false)
                                end
                                Window.RightContent:ChangeItem("GossipSaveChanges", "Disable", false)
                            end,
                            Name = "GossipTBItem",
                            Lines = 3,
                            Value = ""
                        },
                        {
                            Type = "TB",
                            Text = "Note",
                            Func = function(_, Value)
                                tempGossip.Note = Value
                                if SelectedFilterIndex then -- cant undo changes when creating a new item
                                    Window.RightContent:ChangeItem("GossipUndoChanges", "Disable", false)
                                end
                                Window.RightContent:ChangeItem("GossipSaveChanges", "Disable", false)
                            end,
                            Lines = 4,
                            Name = "NoteTBItem",
                            Value = ""
                        },
                        {
                            Type = "CB",
                            Text = "Disable",
                            Func = function(_, Value)
                                if SelectedFilterType == "Preset whitelist" and _Questic.GossipFunctions.DefaultList[SelectedFilterIndex] then
                                    local NPCName = _Questic.GossipFunctions.DefaultList[SelectedFilterIndex].NPC
                                    local Gossip = _Questic.GossipFunctions.DefaultList[SelectedFilterIndex].Gossip
                                    local Categories = _Questic.GossipFunctions.DefaultList[SelectedFilterIndex].Categories
                                    if Value then
                                        _Questic.GetVariable("GossipAutoSelectDisabledPresets")[NPCName .. Gossip .. Categories] = true
                                    else
                                        _Questic.GetVariable("GossipAutoSelectDisabledPresets")[NPCName .. Gossip .. Categories] = nil
                                    end
                                else
                                    tempGossip.Disable = Value
                                    if SelectedFilterIndex then -- cant undo changes when creating a new item
                                        Window.RightContent:ChangeItem("GossipUndoChanges", "Disable", false)
                                    end
                                    Window.RightContent:ChangeItem("GossipSaveChanges", "Disable", false)
                                end
                            end,
                            Name = "DisableCBItem",
                            Value = false
                        },
                        {
                            VSpacing = 6,
                            Type = "DB",
                            Text = "Undo",
                            Name = "GossipUndoChanges",
                            Func = function()
                                UpdateGossipFilterOptionsMenu(SelectedFilterType, SelectedFilterIndex, true)
                            end
                        },
                        {
                            Type = "DB",
                            Text = "Save",
                            Name = "GossipSaveChanges",
                            Func = function()
                                if not tempGossip.NPC or tempGossip.NPC == "" then
                                    print("Questic: Missing NPC name.")
                                    return nil
                                end
                                if not tempGossip.Gossip or tempGossip.Gossip == "" then
                                    print("Questic: Missing gossip.")
                                    return nil
                                end
                                if tempGossip.New then
                                    for i, v in ipairs(_Questic.GetVariable("GossipAutoSelectList")) do
                                        if v.NPC == tempGossip.NPC and v.Gossip == tempGossip.Gossip and (tempGossip.New or not (SelectedFilterType == "User defined whitelist" and SelectedFilterIndex == i)) then
                                            print("Questic: This entry already exists in \"User defined whitelist\"")
                                            return nil
                                        end
                                    end
                                    for i, v in ipairs(_Questic.GetVariable("GossipAutoSelectBlackList")) do
                                        if v.NPC == tempGossip.NPC and v.Gossip == tempGossip.Gossip and (tempGossip.New or not (SelectedFilterType == "User defined blacklist" and SelectedFilterIndex == i)) then
                                            print("Questic: This entry already exists in \"User defined blacklist\"")
                                            return nil
                                        end
                                    end

                                    if tempGossip.Filter == "Whitelist" then
                                        insert(_Questic.GetVariable("GossipAutoSelectList"),
                                            1,
                                            {
                                                NPC = tempGossip.NPC,
                                                Gossip = tempGossip.Gossip,
                                                Note = tempGossip.Note,
                                                Disable = tempGossip.Disable
                                            })
                                        UpdateGossipFilterOptionsMenu("User defined whitelist", 1, true)
                                    else
                                        insert(_Questic.GetVariable("GossipAutoSelectBlackList"), 1, {
                                            NPC = tempGossip.NPC,
                                            Gossip = tempGossip.Gossip,
                                            Note = tempGossip.Note,
                                            Disable = tempGossip.Disable
                                        })
                                        UpdateGossipFilterOptionsMenu("User defined blacklist", 1, true)
                                    end
                                else
                                    if tempGossip.OFilter == tempGossip.Filter then -- no changes made to filter type
                                        if tempGossip.Filter == "Blacklist" and _Questic.GetVariable("GossipAutoSelectBlackList")[SelectedFilterIndex] then
                                            _Questic.GetVariable("GossipAutoSelectBlackList")[SelectedFilterIndex].NPC = tempGossip.NPC
                                            _Questic.GetVariable("GossipAutoSelectBlackList")[SelectedFilterIndex].Gossip = tempGossip.Gossip
                                            _Questic.GetVariable("GossipAutoSelectBlackList")[SelectedFilterIndex].Note = tempGossip.Note
                                            _Questic.GetVariable("GossipAutoSelectBlackList")[SelectedFilterIndex].Disable = tempGossip.Disable
                                        elseif tempGossip.Filter == "Whitelist" and _Questic.GetVariable("GossipAutoSelectList")[SelectedFilterIndex] then
                                            _Questic.GetVariable("GossipAutoSelectList")[SelectedFilterIndex].NPC = tempGossip.NPC
                                            _Questic.GetVariable("GossipAutoSelectList")[SelectedFilterIndex].Gossip = tempGossip.Gossip
                                            _Questic.GetVariable("GossipAutoSelectList")[SelectedFilterIndex].Note = tempGossip.Note
                                            _Questic.GetVariable("GossipAutoSelectList")[SelectedFilterIndex].Disable = tempGossip.Disable
                                        end
                                        UpdateGossipFilterOptionsMenu(SelectedFilterType, SelectedFilterIndex, true)
                                    else -- changes made to filter type
                                        if tempGossip.OFilter == "Blacklist" and _Questic.GetVariable("GossipAutoSelectBlackList")[SelectedFilterIndex] then -- delete item from blacklist
                                            remove(_Questic.GetVariable("GossipAutoSelectBlackList"),
                                                SelectedFilterIndex)
                                        elseif tempGossip.OFilter == "Whitelist" and _Questic.GetVariable("GossipAutoSelectList")[SelectedFilterIndex] then -- delete item from whitelist
                                            remove(_Questic.GetVariable("GossipAutoSelectList"),
                                                SelectedFilterIndex)
                                        end
                                        if tempGossip.Filter == "Blacklist" then -- add item to blacklist
                                            insert(_Questic.GetVariable("GossipAutoSelectBlackList"),
                                                1,
                                                {
                                                    NPC = tempGossip.NPC,
                                                    Gossip = tempGossip.Gossip,
                                                    Note = tempGossip.Note,
                                                    Disable = tempGossip.Disable
                                                })
                                            UpdateGossipFilterOptionsMenu("User defined blacklist", 1, true) -- call item blacklist
                                        elseif tempGossip.Filter == "Whitelist" then -- add item to whitelist
                                            insert(_Questic.GetVariable("GossipAutoSelectList"),
                                                1,
                                                {
                                                    NPC = tempGossip.NPC,
                                                    Gossip = tempGossip.Gossip,
                                                    Note = tempGossip.Note,
                                                    Disable = tempGossip.Disable
                                                })
                                            UpdateGossipFilterOptionsMenu("User defined whitelist", 1, true) -- call item on whitelist
                                        end
                                    end
                                end
                            end
                        },
                        {
                            Type = "DB",
                            Text = "Delete",
                            Name = "GossipDeleteChanges",
                            Func = function()
                                if tempGossip.New then
                                    UpdateGossipFilterOptionsMenu(SelectedFilterType, nil, true) -- clear everything entered for last entry or new entry
                                else
                                    if tempGossip.Filter == "Whitelist" and _Questic.GetVariable("GossipAutoSelectList")[SelectedFilterIndex] then
                                        if Window.VerifyWindow then
                                            Window.VerifyWindow:Release()
                                            Window.VerifyWindow = nil
                                        end
                                        Window.VerifyWindow = Widgets:VerifyAction(UIParent, "Confirm delete", "Are you sure you want to delete: \"" .. _Questic.GetVariable("GossipAutoSelectList")[SelectedFilterIndex].NPC .. ": " .. _Questic.GetVariable("GossipAutoSelectList")[SelectedFilterIndex].Gossip .. "\"?", function()
                                            Window.VerifyWindow = nil
                                            remove(_Questic.GetVariable("GossipAutoSelectList"), SelectedFilterIndex)
                                            UpdateGossipFilterOptionsMenu("User defined whitelist", nil, true)
                                        end)
                                    elseif tempGossip.Filter == "Blacklist" and _Questic.GetVariable("GossipAutoSelectBlackList")[SelectedFilterIndex] then
                                        if Window.VerifyWindow then
                                            Window.VerifyWindow:Release()
                                            Window.VerifyWindow = nil
                                        end
                                        Window.VerifyWindow = Widgets:VerifyAction(UIParent, "Confirm delete", "Are you sure you want to delete: \"" .. _Questic.GetVariable("GossipAutoSelectBlackList")[SelectedFilterIndex].NPC .. ": " .. _Questic.GetVariable("GossipAutoSelectBlackList")[SelectedFilterIndex].Gossip .. "\"?", function()
                                            Window.VerifyWindow = nil
                                            remove(_Questic.GetVariable("GossipAutoSelectBlackList"), SelectedFilterIndex)
                                            UpdateGossipFilterOptionsMenu("User defined blacklist", nil, true)
                                        end)
                                    end
                                end
                            end
                        }
                    })
                end
                UpdateGossipFilterOptionsMenu(SelectedFilterType, nil, true)
            elseif CurrentTab == "RouteDisplay" then
                ----------------------------------------------------------------------------------------------------------------------
                --------------------------------------------Route Display-------------------------------------------------------------
                ----------------------------------------------------------------------------------------------------------------------
                if Row == 3 or Row == 1 then
                    Window.LeftContent:AddItems({
                        {
                            Type = "LB",
                            Text = "Coming soon!"
                        }
                    })
                end
                if Row == 3 or Row == 2 then
                    Window.RightContent:ClearItems()
                end
            elseif CurrentTab == "QuestPlanner" then
                ----------------------------------------------------------------------------------------------------------------------
                --------------------------------------------Route planning------------------------------------------------------------
                ----------------------------------------------------------------------------------------------------------------------
                if Row == 3 or Row == 1 then
                    Window.LeftContent:AddItems({
                        {
                            Type = "LB",
                            Text = "Coming soon!"
                        }
                    })
                end
                if Row == 3 or Row == 2 then
                    Window.RightContent:ClearItems()
                end
            elseif CurrentTab == "Profile" then
                ----------------------------------------------------------------------------------------------------------------------
                ------------------------------------------Profile Settings------------------------------------------------------------
                ----------------------------------------------------------------------------------------------------------------------
                if Row == 3 or Row == 1 then
                    Window.LeftContent:AddItems({
                        {
                            Type = "SL",
                            Height = 250,
                            Text = "Profiles",
                            Name = "ListOfProfiles",
                            List = _Questic.ProfileManagement("GetAll"),
                            Func = function(self, Value, ProfileName)
                                SelectedProfile = ProfileName
                                UpdateProfileTab()
                            end
                        }
                    })
                end
                if Row == 3 or Row == 2 then
                    Window.RightContent:AddItems({
                        {
                            Type = "LB",
                            Text = "Active profile: " .. (_Questic.ProfileManagement("GetActive") or ""),
                            Name = "ActiveProfileLB"
                        },
                        {
                            VSpacing = 10,
                            Type = "DB",
                            Text = "Create new profile",
                            Alignment = "CENTER",
                            Func = function(self)
                                SelectedProfile = _Questic.ProfileManagement("CreateNew")
                                UpdateProfileTab()
                            end
                        },
                        { Type = "LB", Text = "Selected profile: " .. (SelectedProfile or ""), Name = "SelectedProfileLB", VSpacing = 10 },
                        {
                            Type = "DB",
                            Text = "Load",
                            Name = "LoadProfileButton",
                            Alignment = "CENTER",
                            Disable = not SelectedProfile,
                            Func = function(self)
                                SelectedProfile = _Questic.ProfileManagement("Load", SelectedProfile)
                                UpdateProfileTab()
                            end
                        },
                        {
                            VSpacing = 10,
                            Type = "TB",
                            Name = "RenameSelectedProfileTB",
                            Text = "Rename selected profile name",
                            Disable = not SelectedProfile,
                            Tooltip = "",
                            Lines = 1,
                            Func = function(self, Value)
                                SelectedProfile = _Questic.ProfileManagement("Rename", SelectedProfile, Value)
                                UpdateProfileTab()
                            end,
                            Value = SelectedProfile or ""
                        },
                        {
                            VSpacing = 10,
                            Type = "SB",
                            Text = "Copy settings from:",
                            Name = "SelectSourceCopyProfileSB",
                            Disable = not SelectedProfile,
                            Options = function() return _Questic.ProfileManagement("GetAll") end,
                            Value = CopyProfile or "",
                            Func = function(self, Value)
                                CopyProfile = Value
                                UpdateProfileTab()
                            end
                        },
                        {
                            Type = "DB",
                            Text = "Copy profile settings",
                            Alignment = "CENTER",
                            Name = "CopySourceProfileButton",
                            Disable = not (SelectedProfile and CopyProfile),
                            Tooltip = 'Copy settings from "' .. (CopyProfile or "") .. '" to "' .. (SelectedProfile or "") .. '".',
                            Func = function(self)
                                if Window.VerifyWindow then
                                    Window.VerifyWindow:Release()
                                    Window.VerifyWindow = nil
                                end
                                Window.VerifyWindow = Widgets:VerifyAction(UIParent, "Confirm overwrite", "Are you sure you want to copy setting from  \"" .. (CopyProfile or "") .. '" to "' .. (SelectedProfile or "") .. "\"? This will overwrite all settings in \"" .. SelectedProfile .. "\".", function()
                                    Window.VerifyWindow = nil
                                    _Questic.ProfileManagement("Copy", SelectedProfile, CopyProfile)
                                    CopyProfile = nil
                                    UpdateProfileTab()
                                end)
                            end
                        },
                        {
                            VSpacing = 10,
                            Type = "DB",
                            Text = "Delete",
                            Name = "DeleteProfileDB",
                            Alignment = "CENTER",
                            Disable = not SelectedProfile,
                            Func = function(self)
                                if Window.VerifyWindow then
                                    Window.VerifyWindow:Release()
                                    Window.VerifyWindow = nil
                                end
                                Window.VerifyWindow = Widgets:VerifyAction(UIParent, "Confirm delete", "Are you sure you want to delete \"" .. SelectedProfile .. "\" profile?", function()
                                    SelectedProfile = _Questic.ProfileManagement("Delete", SelectedProfile)
                                    CopyProfile = nil
                                    UpdateProfileTab()
                                end)
                            end
                        }
                    })
                end
            elseif CurrentTab == "Credits" then
                ----------------------------------------------------------------------------------------------------------------------
                ----------------------------------------- Profile Settings------------------------------------------------------------
                ----------------------------------------------------------------------------------------------------------------------
                if Row == 3 or Row == 1 or Row == 2 then
                    Window.LeftContent:AddItems({
                        {
                            Type = "LB",
                            Text = [[|cFFFFFFFFAuthor: |rAvyr

|cFFFFFFFFDebugging and testing: |r
Interfektor
Pawloy



Thank you to everyone that provided feedback in person and on the forums.

If you can think of a feature you think Questic is missing or a feature that could be improved you can leave feedback at: |cFF8888FFwow.curseforge.com/projects/questic|r

Thank you again for using Questic.
]],
                            Alignment = "LEFT"
                        }
                    })
                end
                if Row == 3 or Row == 2 then
                    Window.RightContent:AddItems({
                        {
                            Type = "LB",
                            Text = "|cFFFFFFFFData collection: |r",
                            Alignment = "CENTER"
                        },
                        {
                            Type = "LB",
                            Text = [[|cFFFFFFFFGossip data: |r
Interfektor
Pawloy

|cFFFFFFFFQuest data: |r
Bluespike
Gravedigger
Kastalia
Klaynara
Nisen
Pawloy
Prezij]],
                            Alignment = "LEFT"
                        }
                    })
                end
            elseif _Questic.OptionsTabs[CurrentTab] then 
                _Questic.OptionsTabs[CurrentTab](Window,Row)
            else
                ----------------------------------------------------------------------------------------------------------------------
                ----------------------------------------- General Settings-------------------------------------------------------------
                ----------------------------------------------------------------------------------------------------------------------
                if Row == 3 or Row == 1 then
                    Window.LeftContent:AddItems({
                        {
                            Type = "CB",
                            Text = "Enable",
                            Value = _Questic.GetVariable("Enable"),
                            Func = function(self, Value) _Questic.SaveVariable("Enable", Value) end,
                            Tooltip = "Enable or disable all of Questic's functionality."
                        },
                        {
                            Type = "LB",
                            Text = "Questic suppress keys",
                            VSpacing = 10
                        },
                        {
                            Type = "CB",
                            Text = "Enable",
                            Value = _Questic.GetVariable("SuppressKeys"),
                            Func = function(self, Value)
                                _Questic.SaveVariable("SuppressKeys", Value)
                                Window.LeftContent:ChangeItem("ALTSuppressEnable", "Disable", not Value)
                                Window.LeftContent:ChangeItem("SHIFTSuppressEnable", "Disable", not Value)
                                Window.LeftContent:ChangeItem("CTRLSuppressEnable", "Disable", not Value)
                            end,
                            Tooltip = "Suppress Questic (Auto-Accept, turnin and cinematic skip) when any key selected below is pressed."
                        },
                        {
                            Type = "CB",
                            Text = "Alt-key Suppress",
                            Name = "ALTSuppressEnable",
                            HSpacing = 15,
                            Disable = not _Questic.GetVariable("SuppressKeys"),
                            Value = _Questic.GetVariable("AltSuppress"),
                            Func = function(self, Value) _Questic.SaveVariable("AltSuppress", Value) end,
                            Tooltip = "Suppress Questic when the Alt key is pressed."
                        },
                        {
                            Type = "CB",
                            Text = "Shift-key Suppress",
                            Name = "SHIFTSuppressEnable",
                            HSpacing = 15,
                            Disable = not _Questic.GetVariable("SuppressKeys"),
                            Value = _Questic.GetVariable("ShiftSuppress"),
                            Func = function(self, Value) _Questic.SaveVariable("ShiftSuppress", Value) end,
                            Tooltip = "Suppress Questic when the Shift key is pressed."
                        },
                        {
                            Type = "CB",
                            Text = "CTRL-key Suppress",
                            Name = "CTRLSuppressEnable",
                            HSpacing = 15,
                            Disable = not _Questic.GetVariable("SuppressKeys"),
                            Value = _Questic.GetVariable("CTRLSuppress"),
                            Func = function(self, Value) _Questic.SaveVariable("CTRLSuppress", Value) end,
                            Tooltip = "Suppress Questic when the CTRL key is pressed."
                        },
                        {
                            Type = "CB",
                            Text = "Enable minimap button",
                            Value = _Questic.GetVariable("EnableMinimapIcon"),
                            Func = function(self, Value) _Questic.SaveVariable("EnableMinimapIcon", Value) end,
                            Tooltip = "Enable or disable Questic's minimap button."
                        },
                        { Type = "LB", Text = "Cinematic skip" },
                        {
                            Type = "CB",
                            Text = "Enable",
                            Value = _Questic.GetVariable("CinematicSkip"),
                            Func = function(self, Value) _Questic.SaveVariable("CinematicSkip", Value) end,
                            Tooltip = "Automatically skipping cinematics, cutscenes or ingame movies."
                        },
                        { Type = "LB", Text = "Quest log", VSpacing = 20 },
                        {
                            Type = "CB",
                            Text = "Auto track",
                            Value = _Questic.GetVariable("AutoTrack"),
                            Func = function(self, Value) 
                                _Questic.SaveVariable("AutoTrack", Value)
                                Window.LeftContent:ChangeItem("UntrackDungeon", "Disable", not Value)
                                Window.LeftContent:ChangeItem("UntrackRaid", "Disable", not Value) 
                            end,
                            Tooltip = "Track and untrack quests when entering or leaving the relavent zones."
                        },
                        {
                            Type = "CB",
                            Text = "Untrack all in dungeons",
                            Disable = not _Questic.GetVariable("AutoTrack"),
                            Name = "UntrackDungeon",
                            HSpacing = 20,
                            Value = _Questic.GetVariable("UntrackDungeon"),
                            Func = function(self, Value) _Questic.SaveVariable("UntrackDungeon", Value) end,
                            Tooltip = "Untrack (effectively hiding quest log) in dungeons."
                        },
                        {
                            Type = "CB",
                            Text = "Untrack all in raids",
                            Disable = not _Questic.GetVariable("AutoTrack"),
                            Name = "UntrackRaid",
                            HSpacing = 20,
                            Value = _Questic.GetVariable("UntrackRaid"),
                            Func = function(self, Value) _Questic.SaveVariable("UntrackRaid", Value) end,
                            Tooltip = "Untrack (effectively hiding quest log) in raids."
                        }
                    })
                end
                if Row == 3 or Row == 2 then
                    Window.RightContent:AddItems({

                        { Type = "LB", Text = "Gossip/quest quick select window" },
                        {
                            Type = "CB",
                            Lines = 2,
                            Text = "Enable quick select window",
                            Value = _Questic.GetVariable("GossipQuickSelect"),
                            Func = function(self, Value)
                                _Questic.SaveVariable("GossipQuickSelect", Value)
                                Window.RightContent:ChangeItem("GossipQuickSelectScale", "Disable", not Value)
                                Window.RightContent:ChangeItem("QuickGossipPreview", "Disable", not Value)
                                Window.RightContent:ChangeItem("HideBlizzGossipFrame", "Disable", not Value)
                                Window.RightContent:ChangeItem("IncludeQuestsInGosseipSelect", "Disable", not Value)
                                Window.RightContent:ChangeItem("HideBlizzQuestFrame", "Disable", not Value or not _Questic.GetVariable("IncludeQuestsInGosseipSelect"))
                                Window.RightContent:ChangeItem("QuickQuestEnable", "Disable", not Value or not _Questic.GetVariable("IncludeQuestsInGosseipSelect"))
                            end,
                            Tooltip = "Whenever a NPC has gossip options they are displayed next to the cursor for quick selection."
                        },
                        {
                            Type = "CB",
                            Text = "Hide standard gossip frame",
                            Lines = 2,
                            HSpacing = 15,
                            Name = "HideBlizzGossipFrame",
                            Value = _Questic.GetVariable("HideBlizzGossipFrame"),
                            Func = function(self, Value)
                                _Questic.SaveVariable("HideBlizzGossipFrame", Value)
                            end,
                            Tooltip = "Hides the standard (Blizzard) gossip window.",
                            Disable = not _Questic.GetVariable("GossipQuickSelect")
                        },
                        {
                            Type = "CB",
                            Text = "Include quests",
                            HSpacing = 15,
                            Name = "IncludeQuestsInGosseipSelect",
                            Value = _Questic.GetVariable("IncludeQuestsInGosseipSelect"),
                            Func = function(self, Value)
                                _Questic.SaveVariable("IncludeQuestsInGosseipSelect", Value)
                                Window.RightContent:ChangeItem("HideBlizzQuestFrame", "Disable", not Value or not _Questic.GetVariable("GossipQuickSelect"))
                                Window.RightContent:ChangeItem("QuickQuestEnable", "Disable", not Value or not _Questic.GetVariable("GossipQuickSelect"))
                            end,
                            Tooltip = "Includes quests in the gossip quick select menu.",
                            Disable = not _Questic.GetVariable("GossipQuickSelect")
                        },
                        {
                            Type = "CB",
                            HSpacing = 30,
                            Text = "Quick quest",
                            Name = "QuickQuestEnable",
                            Value = _Questic.GetVariable("QuickQuestEnable"),
                            Func = function(self, Value)
                                _Questic.SaveVariable("QuickQuestEnable", Value)
                            end,
                            Tooltip = "Right-click predefined quests to automatically accept and turn them in (for example the seal of fate quests)",
                            Disable = not _Questic.GetVariable("GossipQuickSelect") or not _Questic.GetVariable("IncludeQuestsInGosseipSelect")
                        },
                        {
                            Type = "CB",
                            HSpacing = 30,
                            Text = "Hide default quest frame",
                            Lines = 2,
                            Name = "HideBlizzQuestFrame",
                            Value = _Questic.GetVariable("HideBlizzQuestFrame"),
                            Func = function(self, Value)
                                _Questic.SaveVariable("HideBlizzQuestFrame", Value)
                            end,
                            Tooltip = "Hides the standard (Blizzard) quest window.",
                            Disable = not _Questic.GetVariable("GossipQuickSelect") or not _Questic.GetVariable("IncludeQuestsInGosseipSelect")
                        },
                        {
                            Type = "NB",
                            Text = "Scale",
                            Name = "GossipQuickSelectScale",
                            Tooltip = "Scale of the gossip quick select window.",
                            HSpacing = 15,
                            Max = 3,
                            Min = 0.4,
                            Step = 0.1,
                            Value = _Questic.GetVariable("GossipQuickSelectScale"),
                            Func = function(self, Value)
                                _Questic.SaveVariable("GossipQuickSelectScale", Value)
                            end,
                            Disable = not _Questic.GetVariable("GossipQuickSelect")
                        },
                        {
                            Type = "DB",
                            Text = "Show/hide gossip quick select preview",
                            Lines = 2,
                            Name = "QuickGossipPreview",
                            HSpacing = 15,
                            Tooltip = "Shows or hides a sample gossip frame if no gossip frame is open.",
                            Func = function(self)
                                _Questic.GossipQuickSelect:ShowPreviewFrame();
                            end,
                            Disable = not _Questic.GetVariable("GossipQuickSelect")
                        },
                    })
                end
            end
        end
    end
end


