local _Questic = _Questic_ADDON_Globals;
local GossipInfoNamespace = C_GossipInfo;
local QuestLogNamespace = C_QuestLog;

local QFWidth = 338
local function DisableBlizzQuestFrame(Show)
    QuestFrameGoodbyeButton:EnableMouse(Show);
    QuestFrameCompleteQuestButton:EnableMouse(Show);
    QuestFrameCloseButton:EnableMouse(Show);
    QuestFrameAcceptButton:EnableMouse(Show);
    QuestFrameDeclineButton:EnableMouse(Show);
    QuestFrame:EnableMouse(Show)
    QuestFrame:SetAlpha(Show and 1 or 0);
    if Show then 
        QuestFrame:SetWidth(QFWidth)
    else
        QuestFrame:SetWidth(0.1)
    end 
end

_Questic.QuestFrameSuppress = function()
    if _Questic.IsEnabled() and _Questic.GetVariable("GossipQuickSelect") and _Questic.GetVariable("IncludeQuestsInGosseipSelect") and _Questic.GetVariable("HideBlizzQuestFrame") then
        DisableBlizzQuestFrame(false)
    else
        DisableBlizzQuestFrame(true)
    end
end

_Questic.SelectQuest = function(Index, QuestType, NPCType)
    if QuestType == "Available" then
        SelectAvailableQuest(Index);
        return GossipInfoNamespace.SelectAvailableQuest(Index);
    else
        GossipInfoNamespace.SelectActiveQuest(Index);
        return SelectActiveQuest(Index);
    end
end

_Questic.FetchOfferedQuests = function()
    local ActiveQuestTable = {}
    local AvailableQuestTable = {};
    --title, level, isLowLevel, isComplete, isLegendary, isIgnored
    local activeRaw = GossipInfoNamespace.GetActiveQuests() or {};
    for i = 1, #activeRaw do
        table.insert(ActiveQuestTable, { Index = i, Name = activeRaw[i].title, Complete = activeRaw[i].isComplete, Type = "Gossip", QuestID = activeRaw[i].questID });
    end
    --title, level, isTrivial, frequency, isRepeatable, isLegendary, isIgnored
    local availableRaw = GossipInfoNamespace.GetAvailableQuests() or {};
    for i = 1, #availableRaw do
        table.insert(AvailableQuestTable, { Index = i, Name = availableRaw[i].title, Frequency = availableRaw[i].frequency, Trivial = availableRaw[i].isTrivial, Legendary = availableRaw[i].isLegendary, Type = "Gossip", QuestID = availableRaw[i].questID });
    end
    for i = 1, GetNumActiveQuests() do
        local title, isComplete = GetActiveTitle(i);
        table.insert(ActiveQuestTable, { Index = i, Name = title, Complete = isComplete, Type = "Greeting" });
    end
    for i = 1, GetNumAvailableQuests() do
        local isTrivial, frequency, isRepeatable, isLegendary = GetAvailableQuestInfo(i);
        local title = GetAvailableTitle(i);
        table.insert(AvailableQuestTable, { Index = i, Name = title, Frequency = frequency, Trivial = isTrivial, Legendary = isLegendary, Type = "Greeting" });
    end
    return ActiveQuestTable, AvailableQuestTable;
end

local ToIgnoreTags = {
    128, --Emissary
    140, --Weekly rated award
};

--C_QuestLog.GetInfo(1)
local function QuestDisplayChecker(QuestIndex)
    local info = QuestLogNamespace.GetInfo(QuestIndex)
    if (info.questID == 0) then
        return false
    end
    local QuestTag, _, WQ =  QuestLogNamespace.GetQuestTagInfo(info.questID);
    if type(QuestTag) == "nil" then
        return true;
    elseif WQ then
        return false;
    else
        for i, v in pairs(ToIgnoreTags) do
            if v == QuestTag then
                return false;
            end
        end
        return true;
    end
end

_Questic.ShareQuest = function(QuestID)
    if (QuestID ~= nil) then
        QuestLogNamespace.SetSelectedQuest(QuestID)
        if QuestLogNamespace.IsPushableQuest(QuestID) then
            QuestLogPushQuest();
        end
    end
end


_Questic.GetQuestLogTable = function()
    local QuestTable = {};
    local HeaderID = "Misc";
    for i = 1, QuestLogNamespace.GetNumQuestLogEntries() do
        local info = QuestLogNamespace.GetInfo(i)
        local Pushable = "0";
        if QuestLogNamespace.IsPushableQuest(info.questID) then
            Pushable = "1";
        end
        if info.isHeader then
            QuestTable[info.title] = { Name = info.title, Type = "H", QuestIndex = i, Pushable = Pushable, HeaderID = "", OnMap = info.isOnMap };
            HeaderID = info.title;
        elseif QuestDisplayChecker(i) then
            QuestTable[info.questID] = { Name = info.title, Type = "NQ", QuestIndex = i, Pushable = Pushable, HeaderID = HeaderID, OnMap = info.isOnMap };
        else
            QuestTable[info.questID] = { Name = info.title, Type = "Q", QuestIndex = i, Pushable = Pushable, HeaderID = HeaderID, OnMap = info.isOnMap };
        end
    end
    return QuestTable;
end

local UsedList = {};
_Questic.QuestItemList = {};
local RemoveIndex = {};
_Questic.BagScan = function(BagID)
    table.wipe(_Questic.QuestItemList);
    for BagID = 0, 4 do
        for SlotID = 1, GetContainerNumSlots(BagID) do
            local QuestItem, QuestID, Accepted = GetContainerItemQuestInfo(BagID, SlotID);
            if QuestItem and GetItemSpell(select(7, GetContainerItemInfo(BagID, SlotID))) then
                table.insert(_Questic.QuestItemList, { SlotID = SlotID, BagID = BagID });
            elseif QuestID and not UsedList[QuestID] and _Questic.GetVariable("Enable") and _Questic.GetVariable("AutoAccept") and _Questic.GetVariable("AutoAcceptQuestsInBag") and not Accepted and not QuestLogNamespace.IsComplete(QuestID) then
                if _Questic.GetVariable("AutoAcceptQuestsInBag") and _Questic.GetVariable("AutoAccept") then
                    UseContainerItem(BagID, SlotID);
                    UsedList[QuestID] = true;
                end
            end
        end
    end
end

_Questic.QuestItemBlackList = function(Action, ItemName)
    if Action == "Check" then
        if _Questic.GetVariable("QuestItemBarEnableBlackList") then
            if type(ItemName) == "string" then
                local BlackList = _Questic.GetVariable("QuestItemBlackList");
                if BlackList[ItemName] then
                    return true;
                else
                    return false;
                end
            else
                return true;
            end
        else
            return false;
        end
    elseif Action == "Add" and type(ItemName) == "string" and ItemName ~= "" and GetItemInfo(ItemName) then
        local BlackList = _Questic.GetVariable("QuestItemBlackList");
        if BlackList[ItemName] then
            _Questic.RefreshAddonFrames("QuestItemBlackList")
            print("Questic: \"" .. ItemName .. "\" already in quest item bar blacklist.")
        else
            BlackList[ItemName] = true;
            _Questic.RefreshAddonFrames("QuestItemBlackList")
            print("Questic: \"" .. ItemName .. "\" added to quest item bar blacklist.")
        end
    elseif Action == "Remove" and type(ItemName) == "string" and ItemName ~= "" then
        local BlackList = _Questic.GetVariable("QuestItemBlackList");
        BlackList[ItemName] = nil;
        _Questic.RefreshAddonFrames("QuestItemBlackList")
        print("Questic: \"" .. ItemName .. "\" removed from quest item bar blacklist.")
    end
end

local function GetQuestNameFromID(QuestID)
    GameTooltip:SetOwner(UIParent)
    GameTooltip:SetHyperlink("\124cffffff00\124Hquest:" .. QuestID .. ":-1\124h[DUMMY QUEST]\124h\124r");
    local Name = GameTooltipTextLeft1:GetText() or ""
    GameTooltip:Hide();
    return Name;
end



local QuickQuestList = {
    --[43895] = "Seal for gold",
    --[43892] = function() return "Seal for resources" end,
    --[47851] = function() return "Seal for marks" end,

    --BfA
    [52835] = function() return "exchange 10 [" .. (GetItemInfo("\124cff0070dd\124Hitem:137642::::::::120:::::\124h[Mark of Honor]\124h\124r") or "Loading") .. "] for a [" .. (GetCurrencyInfo("|cffa335ee|Hcurrency:1580:0|h[Valor Points]|h|r") or "Loading") .. "]" end, --Seal of Wartorn Fate: Marks of Honor
    [52839] = function() return "exchange 25 [" .. (GetItemInfo("\124cff0070dd\124Hitem:137642::::::::120:::::\124h[Mark of Honor]\124h\124r") or "Loading") .. "] for a [" .. (GetCurrencyInfo("|cffa335ee|Hcurrency:1580:0|h[Valor Points]|h|r") or "Loading") .. "]" end, --Seal of Wartorn Fate: Additional Marks of Honor
    [52834] = function() return "exchange 2,000 gold for a [" .. (GetCurrencyInfo("|cffa335ee|Hcurrency:1580:0|h[Valor Points]|h|r") or "Loading") .. "]" end, --Seal of Wartorn Fate: Gold
    [52838] = function() return "exchange 5,000 gold for a [" .. (GetCurrencyInfo("|cffa335ee|Hcurrency:1580:0|h[Valor Points]|h|r") or "Loading") .. "]" end, --Seal of Wartorn Fate: Piles of Gold
    [52837] = function() return "exchange 250 [" .. (GetCurrencyInfo("|cffa335ee|Hcurrency:824:0|h[Valor Points]|h|r") or "Loading") .. "] for a [" .. (GetCurrencyInfo("|cffa335ee|Hcurrency:1580:0|h[Valor Points]|h|r") or "Loading") .. "]" end, --Seal of Wartorn Fate: War Resources
    [52840] = function() return "exchange 500 [" .. (GetCurrencyInfo("|cffa335ee|Hcurrency:824:0|h[Valor Points]|h|r") or "Loading") .. "] for a [" .. (GetCurrencyInfo("|cffa335ee|Hcurrency:1580:0|h[Valor Points]|h|r") or "Loading") .. "]" end, --Seal of Wartorn Fate: Stashed War Resources
}

local CompiledQuickQuestList = {};
local ActiveQuest;

_Questic.CompileQuickQuestList = function()
    table.wipe(CompiledQuickQuestList);
    for i, v in pairs(QuickQuestList) do
        if QuestLogNamespace.GetInfo(i) then
        CompiledQuickQuestList[QuestLogNamespace.GetInfo(i)] = v
            end
        end
end

_Questic.IsQuickQuest = function(ID)
    if type(ID) == "number" then
        if QuickQuestList[ID] then
            return QuickQuestList[ID]();
        else
            return nil
        end
    elseif type(ID) == "string" then
        if CompiledQuickQuestList[ID] then
            return CompiledQuickQuestList[ID]()
        else
            return nil
        end
    end
end

_Questic.ClearActiveQuickQuest = function()
    ActiveQuest = nil;
end

_Questic.SetActiveQuickQuest = function(Quest)
    ActiveQuest = Quest;
end

_Questic.GetActiveQuickQuest = function()
    return ActiveQuest or ""
end

local QuestsToBeTracked = {}
_Questic.UpdateQuestLog = function()
    if _Questic.GetVariable("Enable") and _Questic.GetVariable("AutoTrack") then
        if  (_Questic.GetVariable("UntrackDungeon") and select(2,GetInstanceInfo()) == "party") 
        or (_Questic.GetVariable("UntrackRaid") and select(2,GetInstanceInfo()) == "raid") then
            for i = 1, QuestLogNamespace.GetNumQuestLogEntries() do
                QuestLogNamespace.RemoveQuestWatch(i)
            end
        else 
            wipe(QuestsToBeTracked)
            local MapID = C_Map.GetBestMapForUnit("player")
            --[[local ParentMapID = C_Map.GetMapInfo(MapID).parentMapID
            local minX, maxX, minY, maxY = 0,1,0,1
            if type(ParentMapID) == "number" then 
                minX, maxX, minY, maxY = C_Map.GetMapRectOnMap(MapID, ParentMapID)
            end ]]
            if type(MapID) == "number" then 
                local Quests = QuestLogNamespace.GetQuestsOnMap(MapID)
                if type(Quests) == "table" then 
                    for i,v in ipairs(Quests) do 
                        QuestsToBeTracked[v.questID] = {x = v.x, y = v.y} 
                    end 
                end 
            end
            for i = 1, QuestLogNamespace.GetNumQuestLogEntries() do
                local info = QuestLogNamespace.GetInfo(i)
                if QuestsToBeTracked[info.questID]
                    --and QuestsToBeTracked[i].x >= minX and QuestsToBeTracked[i].x <= maxX and QuestsToBeTracked[i].y >= minY and QuestsToBeTracked[i].y <= maxY
                then
                    QuestLogNamespace.AddQuestWatch(i)
                else
                    QuestLogNamespace.RemoveQuestWatch(i)
                end
            end
        end 
    end 
end 
