local Widgets = _Avyr_GUI_Widgets_Lib
local GetCursorPosition, UIParent, tonumber, insert = GetCursorPosition, UIParent, tonumber, table.insert;
local BorderSize, Padding = Widgets.BorderSize, Widgets.Padding;

local function SetValue(self, Value, Increment, UserInput, SlideUpdate)
    if not Value then
        return nil
    end
    local NewValue
    if Increment then
        NewValue = self.Value + Value
    else
        NewValue = Value
    end
    local rmd = NewValue % self.StepSize
    if rmd ~= 0 then
        if rmd / self.StepSize < 0.4999 then
            NewValue = NewValue - rmd
        else
            NewValue = NewValue + self.StepSize - rmd
        end
    end
    if NewValue > self.MaxValue then
        NewValue = self.MaxValue
    elseif NewValue < self.MinValue then
        NewValue = self.MinValue
    end
    --print(NewValue,self.Value,SlideUpdate)
    local PreValue = self.Value
    self.Value = NewValue 
    if NewValue ~= PreValue or (SlideUpdate and SlideUpdate == 2 and self.IgnoreSlideUpdates) then     
        if (NewValue ~= PreValue and (not self.IgnoreSlideUpdates or not SlideUpdate)) or (SlideUpdate and SlideUpdate == 2 and self.IgnoreSlideUpdates) then
            if self.OnValueChanged and UserInput then
                self.OnValueChanged(self, self.Value)
            end
        end 
    end
    local TotalLength = self:GetWidth() - self.SliderButton:GetWidth()
    self.SliderButton:SetPoint("BOTTOMLEFT", self.MinusButton, "TOPLEFT", TotalLength * (self.Value - self.MinValue) / (self.MaxValue - self.MinValue), BorderSize)
    self.NumberText:SetValue(self.Value)
end

local function SetMinStepMax(self, Min, Step, Max)
    self.MaxValue = Max
    self.MinValue = Min
    self.StepSize = Step
    --Force a setvalue refresh cycle
    self.Value = self.Value - 1
    self:SetValue(self.Value + 1)
end

local function InternalSliderMoveFunction(self)
    local MouseX, MouseY = GetCursorPosition()
    local Scale = self:GetEffectiveScale()
    MouseX = MouseX / Scale
    local Value = MouseX - self:GetLeft() - self.SliderButton:GetWidth() / 2
    Value = Value * (self.MaxValue - self.MinValue) / (self:GetWidth() - self.SliderButton:GetWidth()) + self.MinValue
    self:SetValue(Value, nil, true, 1)
end

Widgets.CreateSlider = function(self, Parent)
    local Slider = Widgets:CreateFrame(Parent)

    Slider.PlusButton = Widgets:CreateButton(Slider)
    Slider.PlusButton:Size(20, 20)
    Slider.PlusButton:SetIcon(Widgets.MediaPath.."PlusMask.blp", nil, nil, nil, nil, 1, 0.82, 0, true)
    Slider.PlusButton:SetPoint("BOTTOMRIGHT")
    Slider.PlusButton.OnClick = function(self)
        if not Slider.Disable then
            Slider:SetValue(Slider.StepSize, true, true)
        end
    end
    Slider.PlusButton.ActiveCheck = function(self)
        return not Slider.Disable
    end

    Slider.MinusButton = Widgets:CreateButton(Slider)
    Slider.MinusButton:Size(20, 20)
    Slider.MinusButton:SetIcon(Widgets.MediaPath.."MinusMask.blp", nil, nil, nil, nil, 1, 0.82, 0, true)
    Slider.MinusButton:SetPoint("BOTTOMLEFT")
    Slider.MinusButton.OnClick = function(self)
        if not Slider.Disable then
            Slider:SetValue(-Slider.StepSize, true, true)
        end
    end
    Slider.MinusButton.ActiveCheck = function(self)
        return not Slider.Disable
    end

    Slider.NumberText = Widgets:CreateBasicTextBox(Slider, false)
    Slider.NumberText.NumericInputFilter = true
    Slider.NumberText:SetPoint("BOTTOMLEFT", Slider.MinusButton, "BOTTOMRIGHT", Padding, 0)
    Slider.NumberText:SetPoint("TOPRIGHT", Slider.PlusButton, "TOPLEFT", -Padding, 0)
    Slider.NumberText.ActiveCheck = function(self)
        return not Slider.Disable
    end
    Slider.NumberText.OnValueChanged = function(self, Value)
        if not Slider.Disable then
            Value = tonumber(Value)
            if Value then
                Slider:SetValue(Value, false, true)
            end
        end
    end

    Slider.MouseInteraction = function(self, Up)
        if Up then
            self.MouseDownFeedback = nil
            Slider:SetValue(Slider.Value, false, true, 2)
        else
            if not Slider.Disable then
                local MouseX, MouseY = GetCursorPosition()
                local Scale = self:GetEffectiveScale()
                MouseY = MouseY / Scale
                if MouseY < self.SliderButton:GetTop() and MouseY > self.SliderButton:GetBottom() then
                    self.MouseDownFeedback = InternalSliderMoveFunction
                end
            end
        end
    end
    Slider:MakeInteractable(true);

    local SliderBtnSize = 20 - 2 * BorderSize
    Slider.SliderButton = Widgets:CreateFrame(Slider)
    Slider.SliderButton:Size(SliderBtnSize, SliderBtnSize)
    Slider.SliderButton:SetPoint("BOTTOMLEFT", Slider.MinusButton, "TOPLEFT", 0, BorderSize)
    Slider.SliderButton:Backdrop()
    Slider.SliderButton:Backdrop("Backdrop", 1, 0.82, 0)

    Slider.SlidLine = Widgets:FetchTexture(Slider)
    Slider.SlidLine:SetPoint("LEFT", Slider.MinusButton, "TOPLEFT", SliderBtnSize / 2, BorderSize + SliderBtnSize / 2)
    Slider.SlidLine:SetPoint("RIGHT", Slider.PlusButton, "TOPRIGHT", -SliderBtnSize / 2, BorderSize + SliderBtnSize / 2)
    Slider.SlidLine:SetHeight(2)
    Slider.SlidLine:SetColorTexture(1, 0.82, 0)

    Slider.SetValue = SetValue
    Slider.SetMinStepMax = SetMinStepMax

    Slider.Value = 0
    Slider:SetMinStepMax(0, 0.1, 1)

    Slider:SizeChanged(function()
        Slider:SetValue(self.Value)
    end)

    insert(Slider.LM, "NumberText");
    insert(Slider.LM, "PlusButton");
    insert(Slider.LM, "MinusButton");
    insert(Slider.LM, "SliderButton");
    insert(Slider.LM, "SlidLine");
    insert(Slider.LM, "SetValue");
    insert(Slider.LM, "SetMinStepMax");
    insert(Slider.LM, "OnValueChanged");
    insert(Slider.LM, "MaxValue");
    insert(Slider.LM, "MinValue");
    insert(Slider.LM, "Value");
    insert(Slider.LM, "StepSize");
    insert(Slider.LM, "MouseInteraction");
    insert(Slider.LM, "EnterLeave");
    insert(Slider.LM, "MouseDownFeedback");

    return Slider
end
